@@ -2,8 +2,8 @@
 /*
  * gsf-infile-zip.c :
  *
- * Copyright (C) 2002-2004 Jody Goldberg (jody@gnome.org)
- *                    Tambet Ingo   (tambet@ximian.com)
+ * Copyright (C) 2002-2006 Jody Goldberg (jody@gnome.org)
+ *                    	   Tambet Ingo   (tambet@ximian.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -227,6 +227,9 @@ zip_dirent_new_in (GsfInfileZip *zip, gsf_off_t *offset)
 	dirent->csize =         csize;
 	dirent->usize =         usize;
 	dirent->offset =        off;
+#if 0
+	g_print ("%s = 0x%x @ %" GSF_OFF_T_FORMAT "\n", name, off, *offset);
+#endif
 
 	*offset += ZIP_DIRENT_SIZE + name_len + extras_len + comment_len;
 
@@ -387,20 +390,35 @@ zip_child_init (GsfInfileZip *child, GError **errmsg)
 {
 	static guint8 const header_signature[] =
 		{ 'P', 'K', 0x03, 0x04 };
-	guint8 const *data;
+	guint8 const *data = NULL;
 	guint16 name_len, extras_len;
+	char *err = NULL;
+
 	GsfZipDirent *dirent = child->vdir->dirent;
 
 	/* skip local header
 	 * should test tons of other info, but trust that those are correct
 	 **/
 
-	if (gsf_input_seek (child->source, (gsf_off_t) dirent->offset, G_SEEK_SET) ||
-	    NULL == (data = gsf_input_read (child->source, ZIP_FILE_HEADER_SIZE, NULL)) ||
-	    0 != memcmp (data, header_signature, sizeof (header_signature))) {
+	if (gsf_input_seek (child->source, (gsf_off_t) dirent->offset, G_SEEK_SET))
+		err = g_strdup_printf ("Error seeking to zip header @ %" GSF_OFF_T_FORMAT,
+				       dirent->offset);
+	else if (NULL == (data = gsf_input_read (child->source, ZIP_FILE_HEADER_SIZE, NULL)))
+		err = g_strdup_printf ("Error reading %d bytes in zip header", ZIP_FILE_HEADER_SIZE);
+	else if (0 != memcmp (data, header_signature, sizeof (header_signature))) {
+		err = g_strdup_printf ("Error incorrect zip header @ %" GSF_OFF_T_FORMAT,
+				       dirent->offset);
+		g_print ("Header is :\n");
+		gsf_mem_dump (data, sizeof (header_signature));
+		g_print ("Header should be :\n");
+		gsf_mem_dump (header_signature, sizeof (header_signature));
+	}
+
+	if (NULL != err) {
+		g_print (err);
 		if (errmsg != NULL)
-			*errmsg = g_error_new (gsf_input_error_id (), 0,
-					       "Unable to read zip header.");
+			*errmsg = g_error_new (gsf_input_error_id (), 0, err);
+		g_free (err);
 		return TRUE;
 	}
 
@@ -552,9 +570,8 @@ gsf_infile_zip_seek (GsfInput *input, gsf_off_t offset, GSeekType whence)
 	}
 
 	if (zip->stream) {
-		zip->stream->next_in  = NULL;
-		zip->stream->avail_in = 0;
-		zip->stream->total_in = 0;
+		(void) inflateEnd (zip->stream);
+		memset (zip->stream, 0, sizeof (z_stream));
 	}
 
 	if (zip_child_init (zip, NULL)) {
@@ -831,9 +848,9 @@ gsf_infile_zip_class_init (GObjectClass *gobject_class)
 				      G_PARAM_CONSTRUCT_ONLY));
 }
 
-GSF_CLASS (GsfInfileZip, gsf_infile_zip,
-	   gsf_infile_zip_class_init, gsf_infile_zip_init,
-	   GSF_INFILE_TYPE)
+GSF_DYNAMIC_CLASS (GsfInfileZip, gsf_infile_zip,
+		   gsf_infile_zip_class_init, gsf_infile_zip_init,
+		   GSF_INFILE_TYPE)
 
 /**
  * gsf_infile_zip_new :
