@@ -2,7 +2,7 @@
 /*
  * gsf-output-gzip.h: wrapper to compress to gzipped output
  *
- * Copyright (C) 2002-2004 Jon K Hellan (hellan@acm.org)
+ * Copyright (C) 2002-2006 Jon K Hellan (hellan@acm.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -15,14 +15,14 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
 #ifndef GSF_OUTPUT_GZIP_H
 #define GSF_OUTPUT_GZIP_H
 
-#include "gsf-output.h"
+#include <gsf/gsf-output.h>
 
 G_BEGIN_DECLS
 
@@ -32,9 +32,10 @@ G_BEGIN_DECLS
 
 typedef struct _GsfOutputGZip GsfOutputGZip;
 
-GType gsf_output_gzip_get_type (void);
+GType gsf_output_gzip_get_type      (void) G_GNUC_CONST;
+void  gsf_output_gzip_register_type (GTypeModule *module);
 
-GsfOutputGZip *gsf_output_gzip_new (GsfOutput *source, GError **err);
+GsfOutput *gsf_output_gzip_new	    (GsfOutput *sink, GError **err);
 
 G_END_DECLS
 
