@@ -2,7 +2,7 @@
 /*
  * gsf-structured_blob.c : Utility storage to blob in/out a tree of data
  *
- * Copyright (C) 2002-2003 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -29,7 +29,7 @@
 #include <gsf/gsf-shared-memory.h>
 #include <string.h>
 
-#define GET_CLASS(instance) G_TYPE_INSTANCE_GET_CLASS (instance, GSF_STRUCTURED_BLOB_TYPE, GsfStructuredBlobClass)
+static GObjectClass *parent_class;
 
 struct _GsfStructuredBlob {
 	GsfInfile base;
@@ -45,7 +45,6 @@ static void
 blob_finalize (GObject *obj)
 {
 	unsigned i;
-	GObjectClass *parent_class;
 	GsfStructuredBlob *blob = GSF_STRUCTURED_BLOB (obj);
 
 	if (blob->data != NULL) {
@@ -60,19 +59,15 @@ blob_finalize (GObject *obj)
 		blob->children = NULL;
 	}
 
-	parent_class = g_type_class_peek (GSF_INFILE_TYPE);
-	if (parent_class && parent_class->finalize)
-		parent_class->finalize (obj);
+	parent_class->finalize (obj);
 }
 
 static GsfInput *
-blob_dup (GsfInput *input, GError **err)
+blob_dup (GsfInput *input, G_GNUC_UNUSED GError **err)
 {
 	GsfStructuredBlob const *src = (GsfStructuredBlob *) input;
 	GsfStructuredBlob *dst = g_object_new (GSF_STRUCTURED_BLOB_TYPE, NULL);
 
-	(void)err;
-
 	if (src->data != NULL) {
 		dst->data = src->data;
 		g_object_ref (G_OBJECT (dst->data));
@@ -138,18 +133,18 @@ blob_name_by_index (GsfInfile *infile, int i)
 }
 
 static GsfInput   *
-blob_child_by_index (GsfInfile *infile, int i)
+blob_child_by_index (GsfInfile *infile, int i, GError **err)
 {
 	GsfStructuredBlob const *blob = (GsfStructuredBlob *) infile;
 	if (blob->children != NULL) {
 		g_return_val_if_fail (i < 0 || (unsigned)i >= blob->children->len, NULL);
-		return gsf_input_dup (g_ptr_array_index (blob->children, i), NULL);
+		return gsf_input_dup (g_ptr_array_index (blob->children, i), err);
 	}
 	return NULL;
 }
 
 static GsfInput   *
-blob_child_by_name (GsfInfile *infile, char const *name)
+blob_child_by_name (GsfInfile *infile, char const *name, GError **err)
 {
 	GsfStructuredBlob const *blob = (GsfStructuredBlob *) infile;
 	if (blob->children != NULL) {
@@ -159,7 +154,7 @@ blob_child_by_name (GsfInfile *infile, char const *name)
 		for (i = 0 ; i < blob->children->len ;) {
 			child = g_ptr_array_index (blob->children, i);
 			if (!strcmp (gsf_input_name (child), name))
-				return gsf_input_dup (child, NULL);
+				return gsf_input_dup (child, err);
 		}
 	}
 	return NULL;
@@ -188,17 +183,19 @@ gsf_structured_blob_class_init (GObjectClass *gobject_class)
 	infile_class->name_by_index	= blob_name_by_index;
 	infile_class->child_by_index	= blob_child_by_index;
 	infile_class->child_by_name	= blob_child_by_name;
+
+	parent_class = g_type_class_peek_parent (gobject_class);
 }
 
-GSF_CLASS (GsfStructuredBlob, gsf_structured_blob,
-	   gsf_structured_blob_class_init, gsf_structured_blob_init,
-	   GSF_INFILE_TYPE)
+GSF_DYNAMIC_CLASS (GsfStructuredBlob, gsf_structured_blob,
+		   gsf_structured_blob_class_init, gsf_structured_blob_init,
+		   GSF_INFILE_TYPE)
 
 /**
  * gsf_structured_blob_read :
- * @input : An input (potentially a GsfInfile) holding the blob
+ * @input: An input (potentially a GsfInfile) holding the blob
  *
- * Returns a freshly created tree of blobs
+ * Returns: a freshly created tree of blobs
  **/
 GsfStructuredBlob *
 gsf_structured_blob_read (GsfInput *input)
@@ -259,13 +256,13 @@ gsf_structured_blob_read (GsfInput *input)
 
 /**
  * gsf_structured_blob_write :
- * @blob :
- * @output :
+ * @blob:
+ * @container:
  *
- * Dumps a structured blob onto the @output.  Will fail if the output is not an
- * Outfile and blob has multiple streams.
+ * Dumps structured blob @blob onto the @container.  Will fail if the output is
+ * not an Outfile and blob has multiple streams.
  *
- * Returns : TRUE on success.
+ * Returns: TRUE on success.
  **/
 gboolean
 gsf_structured_blob_write (GsfStructuredBlob *blob, GsfOutfile *container)
